@@ -4,11 +4,11 @@ import os
 import numpy as np
 import dill
 import pathlib
-from datetime import datetime
+from datetime import datetime, timedelta
 from monaco.MCCase import MCCase
 from monaco.MCVar import MCInVar, MCOutVar
 from monaco.MCEnums import MCFunctions, SampleMethod
-from monaco.helper_functions import get_iterable, slice_by_index, vprint, vwarn, vwrite, hash_str_repeatable
+from monaco.helper_functions import get_sequence, slice_by_index, vprint, vwarn, vwrite, hash_str_repeatable
 from psutil import cpu_count
 from pathos.pools import ThreadPool as Pool
 from tqdm import tqdm
@@ -22,13 +22,13 @@ class MCSim:
                  fcns           : dict[MCFunctions, Callable], # fcns is a dict with keys MCFunctions.PREPROCESS, RUN, and POSTPROCESS
                  firstcaseisnom : bool = False, 
                  samplemethod   : SampleMethod = SampleMethod.SOBOL_RANDOM,
-                 seed           : int  = np.random.get_state()[1][0], 
+                 seed           : int  = np.random.get_state(legacy=False)['state']['key'][0], 
                  cores          : int  = cpu_count(logical=False), 
                  verbose        : bool = True,
                  debug          : bool = False,
                  savesimdata    : bool = True,
                  savecasedata   : bool = True,
-                 resultsdir     : Union[None, str, pathlib.Path] = None,
+                 resultsdir     : Union[str, pathlib.Path] = None,
                  ):
         
         self.checkFcnsInput(fcns)
@@ -54,34 +54,35 @@ class MCSim:
             self.resultsdir = self.rootdir / f'{self.name}_results'
         self.filepath = self.resultsdir / f'{self.name}.mcsim'
 
-        self.invarseeds = []
-        self.caseseeds = []
+        self.invarseeds : list[int] = []
+        self.caseseeds  : list[int] = []
         
-        self.inittime = datetime.now()
-        self.starttime = None
-        self.endtime = None
-        self.runtime = None
-        self.casespreprocessed = set()
-        self.casesrun = set()
-        self.casespostprocessed = set()
-        
-        self.mcinvars = dict()
-        self.mcoutvars = dict()
-        self.constvals = dict()
-        self.mccases = []
-        self.ninvars = 0
-        
-        self.corrcoeffs = None
-        self.covs = None
-        self.covvarlist = None
-        
-        self.pbar0 = None
-        self.pbar1 = None
-        self.pbar2 = None
+        self.inittime  : datetime = datetime.now()
+        self.starttime : datetime = None
+        self.endtime   : datetime = None
+        self.runtime   : timedelta = None
 
-        self.runsimid = None
+        self.casespreprocessed  : set[int] = set()
+        self.casesrun           : set[int] = set()
+        self.casespostprocessed : set[int] = set()
+        
+        self.mcinvars  : dict[str, MCInVar] = dict()
+        self.mcoutvars : dict[str, MCOutVar] = dict()
+        self.constvals : dict[str, Any] = dict()
+        self.mccases : list[MCCase] = []
+        self.ninvars : int = 0
+        
+        self.corrcoeffs : np.ndarray = None
+        self.covs       : np.ndarray = None
+        self.covvarlist : list[str] = None
+        
+        self.pbar0 : tqdm = None
+        self.pbar1 : tqdm = None
+        self.pbar2 : tqdm = None
 
-        self.ncases = ndraws + 1
+        self.runsimid : int = None
+
+        self.ncases : int = ndraws + 1
         self.setFirstCaseNom(firstcaseisnom)
         self.setNDraws(self.ndraws) # Will regen runsimid
                 
@@ -100,7 +101,7 @@ class MCSim:
 
     def checkFcnsInput(self,
                        fcns: dict,
-                       ):
+                       ) -> None:
         if set(fcns.keys()) != {MCFunctions.PREPROCESS, MCFunctions.RUN, MCFunctions.POSTPROCESS}:
             raise ValueError(f"MCSim argument {fcns=} must have keys {MCFunctions.PREPROCESS}, {MCFunctions.RUN}, and {MCFunctions.POSTPROCESS}")
         if any(not callable(f) for f in fcns.values()):
@@ -109,7 +110,7 @@ class MCSim:
 
     def setFirstCaseNom(self, 
                         firstcaseisnom : bool,
-                        ):
+                        ) -> None:
         if firstcaseisnom:
            self.firstcaseisnom = True
            self.ncases = self.ndraws + 1
@@ -125,25 +126,27 @@ class MCSim:
                  name       : str, 
                  dist       : Union[rv_discrete, rv_continuous],
                  distkwargs : dict[str, Any], 
-                 nummap     : Union[None, dict[int, Any]] = None,
-                 ):  
+                 nummap     : dict[int, Any] = None,
+                 seed       : int = None,
+                 ) -> None:  
         self.ninvars += 1
-        invarseed = (self.seed + hash_str_repeatable(name)) % 2**32  # make seed dependent on var name and not order added
-        self.invarseeds.append(invarseed)
+        if seed is None:
+            seed = (self.seed + self.ninvars) % 2**32  # seed is dependent order added
+        self.invarseeds.append(seed)
         self.mcinvars[name] = MCInVar(name=name, dist=dist, distkwargs=distkwargs, ndraws=self.ndraws, nummap=nummap, \
-                                      samplemethod=self.samplemethod, ninvar=self.ninvars, seed=invarseed, firstcaseisnom=self.firstcaseisnom, autodraw=False)
+                                      samplemethod=self.samplemethod, ninvar=self.ninvars, seed=seed, firstcaseisnom=self.firstcaseisnom, autodraw=False)
 
 
     def addConstVal(self, 
                     name : str, 
                     val  : Any,
-                    ):  
+                    ) -> None:  
         self.constvals[name] = val
 
 
     def setNDraws(self, 
                   ndraws: int,
-                  ):
+                  ) -> None:
         self.clearResults()
         self.ndraws = ndraws
         self.setFirstCaseNom(self.firstcaseisnom)
@@ -162,13 +165,13 @@ class MCSim:
 
     def runSim(self, 
                cases : Union[None, int, list[int], set[int]] = None,
-               ):
+               ) -> None:
         cases = self.downselectCases(cases=cases)
         vprint(self.verbose, f"Running '{self.name}' Monte Carlo simulation with {len(cases)}/{self.ncases} cases...", flush=True)
         self.runSimWorker(casestogenerate=cases, casestopreprocess=cases, casestorun=cases, casestopostprocess=cases)
 
 
-    def runIncompleteSim(self):
+    def runIncompleteSim(self) -> None:
         casestopreprocess  = self.allCases() - self.casespreprocessed
         casestorun         = self.allCases() - self.casesrun           | casestopreprocess
         casestopostprocess = self.allCases() - self.casespostprocessed | casestopreprocess | casestorun
@@ -186,7 +189,7 @@ class MCSim:
                      casestopreprocess  : Union[None, int, list[int], set[int]],
                      casestorun         : Union[None, int, list[int], set[int]],
                      casestopostprocess : Union[None, int, list[int], set[int]],
-                     ):            
+                     ) -> None:            
         self.starttime = datetime.now()
 
         if casestorun in (None, self.allCases()):
@@ -218,18 +221,18 @@ class MCSim:
             vprint(self.verbose, f"Sim results saved in '{self.filepath}'", flush=True)
 
 
-    def genRunSimID(self):
+    def genRunSimID(self) -> None:
         self.runsimid = self.genID()
 
 
-    def genID(self):
+    def genID(self) -> int:
         uniqueid = (self.seed + hash(self.name) + hash(datetime.now())) % 2**32
         return uniqueid
 
 
     def genCases(self,
                  cases : Union[None, int, list[int], set[int]] = None,
-                 ):
+                 ) -> None:
         vprint(self.verbose, 'Generating cases...', flush=True)
         self.genCaseSeeds()
         
@@ -245,14 +248,14 @@ class MCSim:
         self.mccases.sort(key=lambda mccase: mccase.ncase)
 
     
-    def genCaseSeeds(self):
+    def genCaseSeeds(self) -> None:
         generator = np.random.RandomState(self.seed)
-        self.caseseeds = generator.randint(0, 2**31-1, size=self.ncases)
+        self.caseseeds = list(generator.randint(0, 2**31-1, size=self.ncases))
 
 
     def preProcessCases(self, 
                         cases : Union[None, int, list[int], set[int]],
-                        ):
+                        ) -> None:
         cases = self.downselectCases(cases=cases)
         
         if self.verbose:
@@ -282,7 +285,7 @@ class MCSim:
 
     def preProcessCase(self, 
                        mccase : MCCase,
-                       ):
+                       ) -> MCCase:
         try:
             mccase.siminput = self.fcns[MCFunctions.PREPROCESS](mccase)
             self.casespreprocessed.add(mccase.ncase)
@@ -294,7 +297,7 @@ class MCSim:
             else:
                 vwarn(self.verbose, f'\nPreprocessing case {mccase.ncase} failed')
             
-        if not (self.pbar0 is None):
+        if self.pbar0 is not None:
             self.pbar0.update(1)
             
         return mccase
@@ -302,7 +305,8 @@ class MCSim:
 
     def runCases(self, 
                  cases            : Union[None, int, list[int], set[int]],
-                 calledfromrunsim : bool = False):
+                 calledfromrunsim : bool = False,
+                 ) -> None:
         cases = self.downselectCases(cases=cases)
         
         if not calledfromrunsim:
@@ -338,10 +342,10 @@ class MCSim:
 
     def runCase(self, 
                 mccase : MCCase,
-                ):
+                ) -> None:
         try:
             mccase.starttime = datetime.now()
-            mccase.simrawoutput = self.fcns[MCFunctions.RUN](*get_iterable(mccase.siminput))
+            mccase.simrawoutput = self.fcns[MCFunctions.RUN](*get_sequence(mccase.siminput))
             mccase.endtime = datetime.now()
             mccase.runtime = mccase.endtime - mccase.starttime
             mccase.runsimid = self.runsimid
@@ -361,13 +365,13 @@ class MCSim:
                 raise
             vwrite(self.verbose, f'\nRunning case {mccase.ncase} failed')
         
-        if not (self.pbar1 is None):
+        if self.pbar1 is not None:
             self.pbar1.update(1)
 
 
     def postProcessCases(self, 
                          cases : Union[None, int, list[int], set[int]],
-                         ):
+                         ) -> None:
         cases = self.downselectCases(cases=cases)
         
         if self.verbose:
@@ -397,9 +401,9 @@ class MCSim:
 
     def postProcessCase(self, 
                         mccase : MCCase,
-                        ):
+                        ) -> None:
         try:
-            self.fcns[MCFunctions.POSTPROCESS](mccase, *get_iterable(mccase.simrawoutput))
+            self.fcns[MCFunctions.POSTPROCESS](mccase, *get_sequence(mccase.simrawoutput))
             self.casespostprocessed.add(mccase.ncase)
             mccase.haspostprocessed = True
             
@@ -409,18 +413,18 @@ class MCSim:
             else:
                 vwrite(self.verbose, f'\nPostprocessing case {mccase.ncase} failed')
             
-        if not (self.pbar2 is None):
+        if self.pbar2 is not None:
             self.pbar2.update(1)
             
             
-    def genOutVars(self):
+    def genOutVars(self) -> None:
         for varname in self.mccases[0].mcoutvals.keys():
             vals = []
             for i in range(self.ncases):
                 vals.append(self.mccases[i].mcoutvals[varname].val)
 
             if self.mccases[0].mcoutvals[varname].valmapsource == 'auto':
-                uniquevals = set()
+                uniquevals : set[Any] = set()
                 valmap = None
                 for i in range(self.ncases):
                     if self.mccases[i].mcoutvals[varname].valmap is None:
@@ -439,7 +443,7 @@ class MCSim:
                 self.mccases[i].mcoutvars[varname] = self.mcoutvars[varname]
 
 
-    def genCovarianceMatrix(self):
+    def genCovarianceMatrix(self) -> None:
         self.covvarlist = []
         allnums = []
         for var in self.mcinvars.keys():
@@ -459,17 +463,17 @@ class MCSim:
                                      "This may happen if this variable does not vary, or if an infinite value was drawn.")
 
 
-    def corr(self):
+    def corr(self) -> tuple[np.ndarray, list[str]]:
         self.genCovarianceMatrix()
         return self.corrcoeffs, self.covvarlist
 
 
-    def cov(self):
+    def cov(self) -> tuple[np.ndarray, list[str]]:
         self.genCovarianceMatrix()
         return self.covs, self.covvarlist
 
 
-    def clearResults(self):
+    def clearResults(self) -> None:
         self.mccases = []
         self.mcoutvars = dict()
         self.casespreprocessed = set()
@@ -483,7 +487,7 @@ class MCSim:
         self.runsimid = self.genID()
 
 
-    def reset(self):
+    def reset(self) -> None:
         self.clearResults()
         self.mcinvars = dict()
         self.constvals = dict()
@@ -497,10 +501,10 @@ class MCSim:
                         cases : Union[None, int, list[int], set[int]] = None,
                         ) -> set[int]:
         if cases is None:
-            cases = self.allCases()
+            cases_downselect = self.allCases()
         else:
-            cases = set(get_iterable(cases))
-        return cases
+            cases_downselect = set(get_sequence(cases))
+        return cases_downselect
 
 
     def allCases(self) -> set[int]:
@@ -508,7 +512,7 @@ class MCSim:
         return allCases
 
 
-    def saveSimToFile(self):
+    def saveSimToFile(self) -> None:
         if self.savesimdata:
             self.filepath.unlink(missing_ok = True)
             self.filepath.touch()
@@ -516,7 +520,7 @@ class MCSim:
                 dill.dump(self, file, protocol=dill.HIGHEST_PROTOCOL)
 
 
-    def loadCases(self):
+    def loadCases(self) -> None:
         vprint(self.verbose, f"{self.filepath} indicates {len(self.casesrun)}/{self.ncases} cases were run, attempting to load raw case data from disk...", end='\n', flush=True)
         self.mccases = []
         casesloaded = set()
@@ -546,7 +550,7 @@ class MCSim:
                             if case in self.casespostprocessed:
                                 casesnotpostprocessed.remove(case)
                     except Exception: 
-                        vwarn(f'Unknown error loading {filepath.name}')
+                        vwarn(self.verbose, f'Unknown error loading {filepath.name}')
             except FileNotFoundError:
                 vwarn(self.verbose, f'{filepath.name} expected but not found')
             pbar.update(1)
@@ -583,12 +587,12 @@ class MCSim:
                 filenames.remove(f'{self.name}_{case}.mccase')
             except Exception:
                 pass
+
         return filenames
 
 
-    def removeExtraResultsFiles(self):
+    def removeExtraResultsFiles(self) -> None:
         extrafiles = self.findExtraResultsFiles()
         for file in extrafiles:
             filepath = self.resultsdir / file
             filepath.unlink()
-
